import org.codehaus.groovy.scriptom.ActiveXObject
import org.codehaus.groovy.scriptom.Scriptom

import com.jacob.com.Variant

final def varientNull = Variant.VariantNull

final def isWindows = (System.getProperty('os.name') =~ /(?i)windows/).find()

final def workDir = new File('.').canonicalFile

def exitcode = 0

try {
    Scriptom.inApartment {
        def testDirector = null

        final def stepProps = new Properties();
        final def inputPropsFile = new File(args[0]);
        final def inputPropsStream = null;
        try {
            inputPropsStream = new FileInputStream(inputPropsFile);
            stepProps.load(inputPropsStream);
        }
        catch (IOException e) {
            throw new RuntimeException(e);
        }

        def serverUrl = stepProps['serverUrl']
        def username = stepProps['username']
        def password = stepProps['password'] ? stepProps['password'] : stepProps['passwordscript']
        def domain = stepProps['qcdomain']
        def project = stepProps['qcproject']
        def failmode = stepProps['failmode']
        def issueIds = stepProps['issueIds']?.split(',') as List;
        def expectedStatus = stepProps['expectedState'];

        try {
            println 'Creating connection to Quality Center server';
            testDirector = new ActiveXObject("TDApiOle80.TDConnection")
            testDirector.InitConnectionEx(serverUrl)
            testDirector.Login(username, password)
            testDirector.Connect(domain, project)
            println 'Connection established'

            def bugFactory = testDirector.BugFactory

            def bugFilter = bugFactory.Filter
            def bugpattern = stepProps['bugpattern']

            if (!issueIds) {
                println 'No defects found in source changes!'
            }
            else {
                boolean foundbug = false;
                def issuesXml = new java.io.StringWriter()
                new groovy.xml.MarkupBuilder(issuesXml).issues() {
                    for (def artifactId in issueIds) {
                        try {
                            def artifactData = bugFactory.item(artifactId)
                            if (expectedStatus != artifactData['Status']) {
                                throw new Exception("Status not in expected state! Actual State : " + artifactData['Status'] + "\nExpected State : " + expectedStatus);
                            }
                            println "Status for issue : $artifactId is in correct state!";
                        }
                        catch (Exception e) {
                            e.printStackTrace()
                            if (failmode == "slow") {
                                println "Step fail mode is 'Fail'. Step will fail after processing all defects."
                                exitcode = 1
                            }
                            else if (failmode == "warn") {
                                println "Step fail mode is 'Warn'. Step will complete successfully after processing all defects."
                            }
                            else { // fail-fast is default
                                println "Step fail mode is 'Fail-fast'. Failing step now."
                                throw new Exception("Not all referenced defects were found to report on and step fail mode is 'Fail-fast'.")
                            }
                        }
                    }
                }
            }
        }
        finally {
            if (testDirector) {
                try {
                    if (testDirector.Connected()) {
                        testDirector.Disconnect()
                    }
                    if (testDirector.LoggedIn()) {
                        testDirector.Logout()
                    }
                    testDirector.ReleaseConnection()
                    testDirector.safeRelease()
                }
                catch (Exception e) {}
            }
        }
    }
}
catch (Exception e) {
    println e.getMessage()
    exitcode = 1
}

com.jacob.com.ComThread.Release();

if (exitcode) {
    System.exit(exitcode)
}
