import org.codehaus.groovy.scriptom.ActiveXObject
import org.codehaus.groovy.scriptom.Scriptom

import com.jacob.com.Variant

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.TimeZone;

final def COMMENT_LINE = "<b>________________________________________</b>";
final def FONT_OPEN_ELEMENT = "<font color=\"#000080\">";
final def FONT_CLOSE_ELEMENT = "</font>";
final def HTML_OPEN_ELEMENT = "<html>";
final def HTML_CLOSE_ELEMENT = "</html>";
final def BODY_OPEN_ELEMENT = "<body>";
final def BODY_CLOSE_ELEMENT = "</body>";
final def COMMENT_SEPERATOR = FONT_OPEN_ELEMENT + COMMENT_LINE + FONT_CLOSE_ELEMENT;
final def LESS_THAN_CHAR = "&lt;";
final def GREATER_THAN_CHAR = "&gt;";
final def BREAK_ELEMENT = "<br>";
final def BOLD_OPEN_ELEMENT = "<b>";
final def BOLD_CLOSE_ELEMENT = "</b>";

def exitcode = 0
def buglist = [] as List

try {
	Scriptom.inApartment{
	    def testDirector = null
	    def bugs = [] as List
	    try {
            final def stepProps = new Properties();
            final def inputPropsFile = new File(args[0]);
            final def inputPropsStream = null;
            try {
                inputPropsStream = new FileInputStream(inputPropsFile);
                stepProps.load(inputPropsStream);
            }
            catch (IOException e) {
                throw new RuntimeException(e);
            }

	        def serverUrl = stepProps['serverUrl']
	        def username = stepProps['username']
	        def password = stepProps['password'] ? stepProps['password'] : stepProps['passwordscript']
	        
	        def domain = stepProps['qcdomain']
	        def project = stepProps['qcproject']
			def failmode = stepProps['failmode']
            def issueIds = stepProps['issueIds']?.split(',') as List;
			
			def additionalComments = stepProps['additionalcomments']
			if (additionalComments == null) {
				println "No additional comments defined to add. Nothing to do."
			}
			else {
				println 'Creating connection to Quality Center server'
			    testDirector = new ActiveXObject("TDApiOle80.TDConnection")
			    testDirector.InitConnectionEx(serverUrl)
			    testDirector.Login(username, password)
			    testDirector.Connect(domain, project)
			    println 'Connection established'
				
				def reqNumbugs = 0
				def actNumbugs = 0
				def bugFactory = testDirector.BugFactory
				
				println 'Defect Ids to comment on: ' + issueIds.join(", ")
				reqNumbugs = issueIds.size()
				for (entry in issueIds) {
					def bug = bugFactory.Item(entry)
					if (!bug) {
						println 'Warning: no defect found for id ' + entry
						continue // skip this defect
					}
					try {
					    def testVar = bug['Status']
						bug['Status'] = testVar
					}
					catch(Exception e) {
						println 'Warning: no defect found for id ' + entry
						continue // skip this defect
					}
					bug.UnlockObject()
					buglist.add(bug)
					actNumbugs++
				} // end for entry in id2comment
				
				println "Found $actNumbugs of $reqNumbugs defects in Quality Center."
				
				if (reqNumbugs != actNumbugs) {
					if (failmode == "slow") {
						println "Fail mode is 'Fail' and not all requested defects were found in Quality Center. Scheduling failure after the found defects have been updated."
						exitcode = 1
					}
					else if (failmode == "warn") {
						println "Fail mode is 'Warn' and not all requested defects were found in Quality Center. Only logging a warning."
					}
					else { // default is fast
						println "Fail mode is 'Fail-fast' and not all requested defects were found in Quality Center. Failing now."
						throw new Exception("Not all requested defects were found in Quality Center and fail mode is 'Fail-fast'")
					}
				}
			    
			    for (bug in buglist) {
					println 'Updating comment on defect ' + bug['id']
					
                    String oldcomments = bug.Field['BG_DEV_COMMENTS']
                    String newComment = null
        			
                	def commentBody = additionalComments
        			
        			if (oldcomments != null
        			        && oldcomments.trim().length() > 0
        			        && !oldcomments.trim().equals("null")) {
        				
        				// @see appendComment in the old integration
        				StringBuffer result = new StringBuffer()
        				
        				def insertIndex = oldcomments.indexOf(BODY_CLOSE_ELEMENT)
        				
        				def prefix = oldcomments.substring(0, insertIndex)
        				def suffix = oldcomments.substring(insertIndex)
        				
        				result.append(prefix)
        				result.append(BREAK_ELEMENT)
        				result.append(COMMENT_SEPERATOR)
        				result.append(BREAK_ELEMENT)
        				result.append(createHtmlFormattedComment(username,commentBody))
        				result.append(suffix)
        				
        				newComment = result.toString()
        			} // end appending
        			else { // otherwise, creating
        				StringBuffer result = new StringBuffer();
        				
        				result.append(HTML_OPEN_ELEMENT);
        				result.append(BODY_OPEN_ELEMENT);
        				
        				result.append(createHtmlFormattedComment(username, commentBody));
        				
        				result.append(BODY_CLOSE_ELEMENT);
        				result.append(HTML_CLOSE_ELEMENT);
        				
        				newComment = result.toString();
        			}
        			bug.Field['BG_DEV_COMMENTS'] = newComment
        			
					println 'Posting changes to defect ' + bug['id']
					bug.Post()
					bug.UnlockObject()
					println 'Post complete'
				}
			} 
	    } // end try from up top
	    finally {
	    	if (testDirector) {
		        try {
			        if (testDirector.Connected()) {
			            testDirector.Disconnect()
			        }
			        if (testDirector.LoggedIn()) {
			            testDirector.Logout()
			        }
		            testDirector.ReleaseConnection()
		            testDirector.safeRelease()
		        }
		        catch (Exception e) {}
	        }
	    }
	}
}
catch (Exception e) {
	println e
	exitcode = 1
}
com.jacob.com.ComThread.Release();

if (exitcode) {
	System.exit(exitcode)
}


String createHtmlFormattedComment(String username, String comment) {
	
	// it wasn't finding these... so I made it find them here
	final def COMMENT_LINE = "<b>________________________________________</b>";	
	final def FONT_OPEN_ELEMENT = "<font color=\"#000080\">";
	final def FONT_CLOSE_ELEMENT = "</font>";
	final def HTML_OPEN_ELEMENT = "<html>";
	final def HTML_CLOSE_ELEMENT = "</html>";
	final def BODY_OPEN_ELEMENT = "<body>";
	final def BODY_CLOSE_ELEMENT = "</body>";
	final def COMMENT_SEPERATOR = FONT_OPEN_ELEMENT + COMMENT_LINE + FONT_CLOSE_ELEMENT;
	final def LESS_THAN_CHAR = "&lt;";
	final def GREATER_THAN_CHAR = "&gt;";
	final def BREAK_ELEMENT = "<br>";
	final def BOLD_OPEN_ELEMENT = "<b>";
	final def BOLD_CLOSE_ELEMENT = "</b>";
	
	StringBuffer result  = new StringBuffer();

	Date currentDate = new Date();

	result.append(FONT_OPEN_ELEMENT);
	result.append(BOLD_OPEN_ELEMENT);
	result.append("Anthill3");
	result.append(LESS_THAN_CHAR);
	result.append(username);
	result.append(GREATER_THAN_CHAR);
	result.append(", ");
	result.append(formatDate(currentDate));
	result.append(": ");
	result.append(BOLD_CLOSE_ELEMENT);
	result.append(FONT_CLOSE_ELEMENT);
	result.append(comment);
	
	return result.toString();
}

String formatDate(Date date) {
	SimpleDateFormat dateFormat = new SimpleDateFormat("MM/dd/yyyy");
	dateFormat.setTimeZone(TimeZone.getDefault());
	return dateFormat.format(date);
}
