import org.codehaus.groovy.scriptom.ActiveXObject
import org.codehaus.groovy.scriptom.Scriptom
import java.util.Date

import com.jacob.com.Variant

final def isWindows = (System.getProperty('os.name') =~ /(?i)windows/).find()
final def toolcmd = isWindows ? 'ahptool.cmd' : 'ahptool'

final def workDir = new File('.').canonicalFile

Scriptom.inApartment{
    def testDirector = null
    try {
        final def stepProps = new Properties();
        final def inputPropsFile = new File(args[0]);
        final def inputPropsStream = null;
        try {
            inputPropsStream = new FileInputStream(inputPropsFile);
            stepProps.load(inputPropsStream);
        }
        catch (IOException e) {
            throw new RuntimeException(e);
        }

        def serverUrl = stepProps['serverUrl']
        def username = stepProps['username']
        def password = stepProps['password'] ? stepProps['password'] : stepProps['passwordscript']
        
        def domain = stepProps['qcdomain']
        def project = stepProps['qcproject']
        def folder = stepProps['folder']
        def testSetName = stepProps['testSetName']
        def reportName = stepProps['reportName']
        def outFile = stepProps['outFile'];
        
        println 'Creating connection to Quality Center server'
        testDirector = new ActiveXObject("TDApiOle80.TDConnection")
        testDirector.InitConnectionEx(serverUrl)
        testDirector.Login(username, password)
        testDirector.Connect(domain, project)
        println 'Connection established'
        
        def testSetTreeManager = testDirector.TestSetTreeManager
        def testSetFolder = null;
        
        testSetFolder = testSetTreeManager.NodeByPath(folder)
        if (testSetFolder == null) {
            throw new NullPointerException("Could not find the test set folder '${folder}'.")
        }
        
        def testSetList = testSetFolder.FindTestSets(testSetName)
        println "Found ${testSetList.Count} Test Sets"
        
        if (testSetList.Count == 0) {
            throw new Exception("Test Set named ${testSetName} not found.")
        }
        if (testSetList.Count > 1) {
            throw new Exception("Multiple Test Sets named ${testSetName} were found.")
        }
        
        def testSetTestsList = testSetList.Item(1).TSTestFactory.NewList("")
        println "Found ${testSetTestsList.Count} tests"
        int failures = 0
        int successes = 0
        println "Counting pass/fail"
        for (i in 0..<testSetTestsList.Count) {
            def currTest = testSetTestsList.Item(i+1)
            if (currTest.LastRun().Status.equalsIgnoreCase("Passed")) successes++;
            if (currTest.LastRun().Status.equalsIgnoreCase("Failed")) failures++;
        }
        println "Done counting pass/fail ($successes passed, $failures failed) - about to generate test results XML"
        def reportXml = new java.io.StringWriter()
        new groovy.xml.MarkupBuilder(reportXml).
        "test-report"("name": reportName,
                      "type": "HP Quality Center",
                      "job-id": System.getenv('AH_JOB_ID'),
                      "successes": successes,
                      "failures": failures) {
            "test-suite"("name": testSetName,
                         "successes":successes,
                         "failures":failures) {
                for (i in 0..<testSetTestsList.Count) {
                    def currTest = testSetTestsList.Item(i+1)
                    def lastRun = currTest.LastRun()
                    String status = (String)(currTest.Status)
                    if(status.equalsIgnoreCase("Passed")) {
                        status = "success"
                    }
                    else if(status.equalsIgnoreCase("Failed")) {
                        status = "failure"
                    }
                    else {
                        status = "error"
                    }
                    
                    def steps = lastRun.StepFactory.NewList("")
                    
                    def duration = lastRun.Field("RN_DURATION")
                    if (duration != null) {
                        duration = duration * 1000
                    }
                    
                    "test"("name":currTest.Name, "result":status, "time":duration) {
                        "message"("Host: " + lastRun.Field("RN_HOST")){}
                        for (j in 0..<steps.Count) {
                            def step = steps.Item(j+1)
                            "message"("Step ${step.Name} ${step.Status}"){}
                        } // for step
                    } // for test
                } // for test-set
            } // test-suite
        } // test-report
        
        File outputFile = new File(outFile);
        outputFile << reportXml;
        println("Quality Center results outputed successfully")
    } // end try
    catch (Exception e) {
        e.printStackTrace()
        throw e
    }
    finally {
        if (testDirector.Connected()) {
            testDirector.Disconnect()
        }
        if (testDirector.LoggedIn()) {
            testDirector.Logout()
        }
        
        try {
            testDirector.ReleaseConnection()
        }
        catch (Exception e) {}
        
        try {
            testDirector.safeRelease()
        }
        catch (Exception e) {}
    }
}
com.jacob.com.ComThread.Release();
