import org.codehaus.groovy.scriptom.ActiveXObject
import org.codehaus.groovy.scriptom.Scriptom

import com.jacob.com.Variant

final def varientNull = Variant.VariantNull

final def isWindows = (System.getProperty('os.name') =~ /(?i)windows/).find()

final def workDir = new File('.').canonicalFile

def exitcode = 0

try {
    Scriptom.inApartment {
        def testDirector = null

        final def stepProps = new Properties();
        final def inputPropsFile = new File(args[0]);
        final def inputPropsStream = null;
        try {
            inputPropsStream = new FileInputStream(inputPropsFile);
            stepProps.load(inputPropsStream);
        }
        catch (IOException e) {
            throw new RuntimeException(e);
        }
        
        def serverUrl = stepProps['serverUrl']
        def username = stepProps['username']
        def password = stepProps['password'] ? stepProps['password'] : stepProps['passwordscript']
        def domain = stepProps['qcdomain']
        def project = stepProps['qcproject']
        def failmode = stepProps['failmode']
        def matchCriteria = stepProps['matchcriteria']?.split('\n') as List
        def returnFields = stepProps['returnfields']?.split('\n') as List
        
        try {
            println 'Creating connection to Quality Center server';
            testDirector = new ActiveXObject("TDApiOle80.TDConnection")
            testDirector.InitConnectionEx(serverUrl)
            testDirector.Login(username, password)
            testDirector.Connect(domain, project)
            println 'Connection established'
            
            def bugFactory = testDirector.BugFactory
            def bugFilter = bugFactory.Filter
            def bugFields = bugFactory.Fields
            
            if (!matchCriteria) {
                println "No query terms inputed!"
            }
            else if (!returnFields) {
                println "No return fields inputed! No data will be returned"
            }
            else {
                def returnNames = [:]
                for (field in bugFields) {
                    def name = field.name
                    def label = field.Property.UserLabel
                    for (criteria in matchCriteria) {
                        def criterias = criteria.split("=", 2);
                        if (criterias[0].trim().equalsIgnoreCase(label)) {
                            bugFilter.Filter[name] = criterias[1].trim()
                        }
                    }
                    for (returnField in returnFields) {
                        if (returnField.trim().equalsIgnoreCase(label)) {
                            returnNames[name] = returnField
                        }
                    }
                }
                def bugList = bugFilter.NewList()
                println bugList.Count + " defects found!"
                for (bug in bugList) {
                    def returnSet = returnNames.entrySet()
                    returnSet.each{
                        print it.value + ": " + bug.Field[it.key] + ", "
                    }
                    println ""
                }
            }
        }
        finally {
            if (testDirector) {
                try {
                    if (testDirector.Connected()) {
                        testDirector.Disconnect()
                    }
                    if (testDirector.LoggedIn()) {
                        testDirector.Logout()
                    }
                    testDirector.ReleaseConnection()
                    testDirector.safeRelease()
                }
                catch (Exception e) {}
            }
        }
    }
}
catch (Exception e) {
    println e.getMessage()
    exitcode = 1
}
com.jacob.com.ComThread.Release();

if (exitcode) {
    System.exit(exitcode)
}
